#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.meta
~~~~~~~~~~~~~~

Classes and functions for metaprogramming
"""

from __future__ import absolute_import

from .meta import DynamicProperty, setdynattr
