#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.user
~~~~~~~~~~~~~~

Extra python libraries for scaffolding server scripts.
"""

from __future__ import absolute_import

from .user import get_current_username, get_current_groupname
