#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.terminal
~~~~~~~~~~~~~~~~~~

Functions for coloring and managing the terminal.
"""

from __future__ import absolute_import

from . import ansi
from . import colors
