#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.domain
~~~~~~~~~~~~~~~~

Classes and functions for manipulating domains.
"""

from __future__ import absolute_import

from .domain import Domain, get_server_ip
