#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ext_pylib.password
~~~~~~~~~~~~~~~~~~

Module for working with passwords.
"""

from __future__ import absolute_import

from .password import generate_pw
