#!/usr/bin/env python
# -*- coding: utf-8 -*-

#    /''         |''\     |    |
#    \.     |    |../     | *  |..
#    /   \/ T    |   \  / | |  |  \
#    \.../\.| __ |    \/  | |  |../
#  ###################/###############
#                    /

"""
ext_pylib
~~~~~~~~~

Extra python libraries for scaffolding server scripts.
"""

from __future__ import absolute_import

from . import domain
from . import files
from . import input  # pylint: disable=redefined-builtin
from . import password
from . import terminal
from . import user


__title__ = 'ext_pylib'
__version__ = '0.1'
__author__ = 'Harold Bradley III'
__license__ = 'MIT'
__copyright__ = 'Copyright 2015-2016 Harold Bradley III'

# Soli Deo gloria. <><
